import asyncio
import functools
import multiprocessing

import tqdm
import tqdm.asyncio


class Iterator:
    """
    Iterator

    """

    def __init__(self, func, total=None, **kwargs):
        self.func = func
        self.total = total
        self.dynamic_args = None
        self.static_args = None
        self.dynamic_kwargs = None
        self.static_kwargs = None
        self.partial_func = None

    def parse(self, *args, **kwargs):
        """parse"""

        # infer the number of expected iterations from args
        if self.total is None:
            for arg in args:
                if isinstance(arg, list):
                    self.total = len(arg)

        # infer the number of expected iterations from kwargs
        if self.total is None:
            for _, value in kwargs.items():
                if isinstance(value, list):
                    self.total = len(value)

        assert self.total is not None, 'Can not infer the number of expected iterations'

        # parse args
        self.dynamic_args = [[] for _ in range(self.total)]
        self.static_args = []
        for arg in args:
            if isinstance(arg, list):
                assert len(arg) == self.total, f'{len(arg)} != {self.total}'
                for i, val in enumerate(arg):
                    self.dynamic_args[i].append(val)
            else:
                self.static_args.append(arg)

        # parse kwargs
        self.dynamic_kwargs = [{} for _ in range(self.total)]
        self.static_kwargs = {}
        for key, value in kwargs.items():
            if isinstance(value, list):
                assert len(value) == self.total, f'{len(value)} != {self.total}'
                for i, val in enumerate(value):
                    self.dynamic_kwargs[i][key] = val
            else:
                self.static_kwargs[key] = value

        # fix the static args
        self.partial_func = functools.partial(self.func, *self.static_args, **self.static_kwargs)

    def run(self, *args, **kwargs):
        """run"""
        raise NotImplementedError


class SingleProcess(Iterator):
    """
    SingleProcess

    """

    def __init__(self, process, total=None, **kwargs):
        super().__init__(process, total)

    def run(self, *args, **kwargs):
        """run - Please ensure that static arguments precede dynamic arguments"""

        # parse
        self.parse(*args, **kwargs)

        # run
        ret_list = []
        for args, kwargs in tqdm.tqdm(zip(self.dynamic_args, self.dynamic_kwargs),
                                      total=self.total, desc=self.func.__name__):
            ret_list.append(self.partial_func(*args, **kwargs))
        return ret_list


class MultiProcess(Iterator):
    """
    MultiProcess

    """

    def __init__(self, process, total=None, nprocs=multiprocessing.cpu_count(), **kwargs):
        super().__init__(process, total)

        self.nprocs = nprocs if nprocs > 0 else multiprocessing.cpu_count()

    def __call__(self, args):
        return self.partial_func(*args[0], **args[1])

    def run(self, *args, **kwargs):
        """run - Please ensure that static arguments precede dynamic arguments"""

        # parse
        self.parse(*args, **kwargs)

        # run
        with multiprocessing.Pool(self.nprocs) as p:
            ret_list = list(tqdm.tqdm(p.imap(self, zip(self.dynamic_args, self.dynamic_kwargs)),
                                      total=self.total, desc=self.func.__name__))
        return ret_list


class AsyncProcess(Iterator):
    """
    AsyncProcess

    """

    def __init__(self, process, total=None, **kwargs):
        super().__init__(process, total)

        self.ret_list = None

    def __call__(self, args):
        return self.partial_func(*args[0], **args[1])

    async def main(self, loop):
        tasks = [loop.create_task(self(args)) for args in zip(self.dynamic_args, self.dynamic_kwargs)]
        for f in tqdm.asyncio.tqdm.as_completed(tasks, total=self.total, desc=self.func.__name__):
            await f
        self.ret_list = [task.result() for task in tasks]

    def run(self, *args, **kwargs):
        """run - Please ensure that static arguments precede dynamic arguments"""

        # parse
        self.parse(*args, **kwargs)

        # run
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.run_until_complete(self.main(loop))
        return self.ret_list



import os
import glob
import imageio.v2 as iio
import argparse
from lh_tool.Iterator import SingleProcess, MultiProcess


def image2gif(image_path, gif_file, postfix, duration, speed=1):
    image_file_list = sorted(glob.glob(os.path.join(image_path, f'*.{postfix}')))
    if len(image_file_list) == 0:
        return
    gif_file = os.path.abspath(image_path) + '.gif' if gif_file is None else gif_file

    image_file_list = [image_file_list[i] for i in range(0, len(image_file_list), speed)]
    images = [iio.imread(image_file) for image_file in image_file_list]
    duration /= len(image_file_list) - 1
    iio.mimsave(gif_file, images, duration=duration)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, default='.', help='path of image files')
    parser.add_argument('-o', '--output', type=str, help='output gif file')
    parser.add_argument('-p', '--postfix', type=str, default='png', help='postfix of image filename')
    parser.add_argument('-d', '--duration', type=float, default=1, help='duration')
    parser.add_argument('-a', '--speed', type=int, default=1, help='speed for video')
    parser.add_argument('-r', '--recursive', action='store_true', help='convert images to gif recursively')
    parser.add_argument('-n', '--nprocs', type=int, default=1, help='number of process')
    opts = parser.parse_args()
    print(opts)

    try:
        image_path = opts.input
        gif_file = opts.output
        postfix = opts.postfix
        duration = opts.duration
        speed = opts.speed
        recursive = opts.recursive
        nprocs = opts.nprocs
        if recursive:
            image_path_list = glob.glob(os.path.join(opts.input, '*/'))
            if nprocs == 1:
                iterator = SingleProcess(image2gif)
            else:
                iterator = MultiProcess(image2gif, nprocs=nprocs)
            iterator.run(image_path_list, gif_file=None, postfix=postfix, duration=duration, speed=speed)
        else:
            image2gif(image_path, gif_file, postfix, duration, speed)

    except AssertionError as e:
        print(e)


if __name__ == '__main__':
    main()



from lh_tool.Email import CustomEmail


def test_email():
    print('Test lh_tool.Email')
    custom_email = CustomEmail(
        subject='Email: Information',
        content='This is a tests E-mail')
    custom_email.send()


if __name__ == '__main__':
    test_email()



import os
import cv2
import glob
import tqdm
import argparse
from lh_tool.Iterator import SingleProcess, MultiProcess
import imageio.v2 as iio


def images2video(image_path, video_file, postfix, fourcc, fps, frameSize=None, speed=1):
    image_file_list = sorted(glob.glob(os.path.join(image_path, f'*.{postfix}')))
    if len(image_file_list) == 0:
        return
    video_file = os.path.abspath(image_path) + '.mp4' if video_file is None else video_file

    if frameSize is None:
        image = iio.imread(image_file_list[0])
        frameSize = (image.shape[1], image.shape[0])
    videoWriter = cv2.VideoWriter(video_file, fourcc, fps, frameSize)
    assert videoWriter.isOpened(), f'Failed to create file: {video_file}'

    image_file_list = [image_file_list[i] for i in range(0, len(image_file_list), speed)]
    for image_file in tqdm.tqdm(image_file_list, desc=video_file):
        image = iio.imread(image_file)
        if frameSize != (image.shape[1], image.shape[0]):
            image = cv2.resize(image, tuple(frameSize))
        videoWriter.write(image)
    videoWriter.release()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, default='.', help='path of image files')
    parser.add_argument('-o', '--output', type=str, help='output video file')
    parser.add_argument('-p', '--postfix', type=str, default='png', help='postfix of image filename')
    parser.add_argument('-f', '--fps', type=float, default=29.97, help='desired fps for video')
    parser.add_argument('-s', '--size', type=int, nargs=2, help='desired frame size for video')
    parser.add_argument('-a', '--speed', type=int, default=1, help='speed for video')
    parser.add_argument('-r', '--recursive', action='store_true', help='convert video to images recursively')
    parser.add_argument('-n', '--nprocs', type=int, default=1, help='number of process')
    opts = parser.parse_args()
    print(opts)

    try:
        image_path = opts.input
        video_file = opts.output
        postfix = opts.postfix
        fps = opts.fps
        size = None if opts.size is None else tuple(opts.size)
        speed = opts.speed
        recursive = opts.recursive
        nprocs = opts.nprocs
        fourcc = cv2.VideoWriter.fourcc('m', 'p', '4', 'v')
        if recursive:
            image_path_list = glob.glob(os.path.join(opts.input, '*/'))
            if nprocs == 1:
                iterator = SingleProcess(images2video)
            else:
                iterator = MultiProcess(images2video, nprocs=nprocs)
            iterator.run(image_path_list, video_file=None, postfix=postfix,
                         fourcc=fourcc, fps=fps, frameSize=size, speed=speed)
        else:
            images2video(image_path, video_file, postfix, fourcc, fps, size, speed)

    except AssertionError as e:
        print(e)


if __name__ == '__main__':
    main()



import asyncio
from src.lh_tool.Iterator import SingleProcess, MultiProcess, AsyncProcess


def process(a, b, opt='+'):
    if opt == '+':
        return a + b
    else:
        return a - b


async def async_process(a, b, opt='+'):
    await asyncio.sleep(1)
    if opt == '+':
        return a + b
    else:
        return a - b


def test_iterator():
    print('Test lh_tool.MultiProcess')
    a = [i for i in range(10)]
    b = [i for i in range(10)]
    result_list = SingleProcess(process).run(a, b, opt='+')
    print(result_list)

    result_list = MultiProcess(process, nprocs=1).run(a=a, b=b, opt='+')
    print(result_list)

    result_list = AsyncProcess(async_process).run(a=a, b=b, opt='+')
    print(result_list)


if __name__ == '__main__':
    test_iterator()



import os
import cv2
import glob
import argparse
from lh_tool.Iterator import SingleProcess, MultiProcess
import imageio.v2 as iio


def image2image(input_file, output_file, image_size=None):
    image = iio.imread(input_file)
    if image_size is not None:
        image = cv2.resize(image, tuple(image_size))
    iio.imwrite(output_file, image)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, default='.', help='path of input image files')
    parser.add_argument('-o', '--output', type=str, help='path of output image files. The default is the same as the '
                                                         'path of input image files')
    parser.add_argument('-p', '--input_postfix', type=str, default='png', help='original postfix of image filename')
    parser.add_argument('-d', '--output_postfix', type=str, default='jpg', help='desired postfix of image filename')
    parser.add_argument('-s', '--size', type=int, nargs=2, help='desired image size')
    parser.add_argument('-r', '--recursive', action='store_true', help='convert video to images recursively')
    parser.add_argument('-n', '--nprocs', type=int, default=1, help='number of process')
    opts = parser.parse_args()
    print(opts)

    try:
        input_path = opts.input
        output_path = opts.output
        input_postfix = opts.input_postfix
        output_postfix = opts.output_postfix
        image_size = None if opts.size is None else tuple(opts.size)
        recursive = opts.recursive
        nprocs = opts.nprocs
        if recursive:
            input_file_list = glob.glob(os.path.join(input_path, '**', f'*.{input_postfix}'), recursive=True)
        else:
            input_file_list = glob.glob(os.path.join(input_path, f'*.{input_postfix}'))
        output_file_list = []
        for input_file in input_file_list:
            if output_path is not None:
                filename = os.path.basename(input_file)
                filename = os.path.splitext(filename)[0] + f'.{output_postfix}'
                output_file = os.path.join(output_path, filename)
            else:
                output_file = os.path.splitext(input_file)[0] + f'.{output_postfix}'
            output_file_list.append(output_file)

        if nprocs == 1:
            iterator = SingleProcess(image2image)
        else:
            iterator = MultiProcess(image2image, nprocs=nprocs)
        iterator.run(input_file_list, output_file_list, image_size=image_size)

    except AssertionError as e:
        print(e)


if __name__ == '__main__':
    main()



import os
import cv2
import glob
import tqdm
import argparse
from lh_tool.Iterator import SingleProcess, MultiProcess
import imageio.v2 as iio


def video2images(video_file, image_path, postfix, frameSize=None):
    assert os.path.isfile(video_file), f'\'{video_file}\' is not a file'
    image_path = os.path.splitext(video_file)[0] if image_path is None else image_path
    os.makedirs(image_path, exist_ok=True)

    videoCapture = cv2.VideoCapture(video_file)
    assert videoCapture.isOpened(), f'Failed to open file: \'{video_file}\''

    frame_count = int(videoCapture.get(cv2.CAP_PROP_FRAME_COUNT))
    for index in tqdm.trange(frame_count, desc=video_file):
        status, image = videoCapture.read()
        if not status:
            break
        if frameSize is not None:
            image = cv2.resize(image, tuple(frameSize))
        image_file = os.path.join(image_path, f'{index + 1:06d}.{postfix}')
        iio.imwrite(image_file, image)

    videoCapture.release()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, default='.', help='input video file or input path')
    parser.add_argument('-o', '--output', type=str, help='output path')
    parser.add_argument('-p', '--postfix', type=str, default='png', help='postfix of image filename')
    parser.add_argument('-s', '--size', type=int, nargs=2, help='desired frame size for image')
    parser.add_argument('-r', '--recursive', action='store_true', help='convert video to images recursively')
    parser.add_argument('-n', '--nprocs', type=int, default=1, help='number of process')
    opts = parser.parse_args()
    print(opts)

    try:
        video_file = opts.input
        image_path = opts.output
        postfix = opts.postfix
        size = None if opts.size is None else tuple(opts.size)
        recursive = opts.recursive
        nprocs = opts.nprocs
        if recursive:
            video_file_list = glob.glob(os.path.join(opts.input, '*.mp4'))
            if nprocs == 1:
                iterator = SingleProcess(video2images)
            else:
                iterator = MultiProcess(video2images, nprocs=nprocs)
            iterator.run(video_file_list, image_path=None, postfix=postfix, frameSize=size)
        else:
            video2images(video_file, image_path, postfix, size)

    except AssertionError as e:
        print(e)


if __name__ == '__main__':
    main()
